/*======================================*/
/* Audine DLL (file audine.dll)         */
/*======================================*/
/* Public Version 1.2                   */
/* November 24, 2002                    */
/* (c) Christian Buil                   */
/*======================================*/
/* Web: http://www.astrosurf.com/audine */
/* Email : christian.buil@wanadoo.fr    */
/*======================================*/
#include <windows.h>  
#include <stdlib.h>
#include <stdio.h>
#include <io.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <conio.h>
#include <time.h>

#define CTRL_RTC        0x70
#define DATA_RTC        0x71

#define SECOND          0x00
#define MINUTE          0x02
#define HOUR            0x04
#define DAY             0x07
#define DAYOFWEEK       0x06
#define MONTH           0x08
#define YEAR            0x09
#define CENTURY         0x32

#define REGISTER_A      0x0A
#define REGISTER_B      0x0B
#define REGISTER_C      0x0C
#define REGISTER_D      0x0D

#define MASK_UIP        0x80    /* 1 = Heure en cours d'actualisation */
#define MASK_12_24      0x02    /* 0 = 12 heures - 1 = 24 heures      */
#define MASK_BCD        0x04    /* 0 = BCD       - 1 = binaire        */
#define MASK_UPDATE     0x80    /* 1 = Actualiser l'heure             */
#define MASK_BATTERY    0x80    /* 0 = Batteries  plat               */

#define BCD2BIN(n)      (((n)>>4)*10 + ((n)&0x0F))

#define PB  1
#define OK  0

void fast_line_inv(short base,short KAF);
void zi_zh_inv(short base);
void read_pel_fast_inv(short base);
void update_clock (void);

/**************** USAGE (VisualBasic example) *********************
'====================
' Declaration section
'====================
Public Declare Function get_port _
  Lib "AUDINE.DLL" (ByVal n As Integer) As Integer

Public Declare Function read_win_audine_inv _
  Lib "AUDINE.DLL" (ByVal KAF As Integer, ByVal x1 As Integer, ByVal x2 As Integer, _
  ByVal y1 As Integer, ByVal y2 As Integer, _
  ByVal binx As Integer, ByVal biny As Integer, _
  ByVal overscan As Integer, ByVal base As Integer, _
  ByVal flagsystem As Integer, ByRef buf As Integer) As Integer

Public Declare Function fast_vidage_inv _
  Lib "AUDINE.DLL" (ByVal base As Integer, ByVal KAF As Integer) As Integer

Public Declare Function test_0 _
  Lib "AUDINE.DLL" (ByVal base As Integer) As Integer

Public Declare Function test_255 _
  Lib "AUDINE.DLL" (ByVal base As Integer) As Integer

Public Declare Function deconvflat _
  Lib "AUDINE.DLL" (ByVal imax As Integer, ByVal jmax As Integer, _
  ByVal coef As Double, ByRef buf As Integer) As Integer
  
Public Declare Function test_c_0 _
  Lib "AUDINE.DLL" (ByVal base As Integer) As Integer

Public Declare Function test_c_255 _
  Lib "AUDINE.DLL" (ByVal base As Integer) As Integer

Public Declare Function ampli_on _
  Lib "AUDINE.DLL" (ByVal base As Integer) As Integer

Public Declare Function ampli_off _
  Lib "AUDINE.DLL" (ByVal base As Integer) As Integer

Public Declare Function obtu_on _
  Lib "AUDINE.DLL" (ByVal base As Integer) As Integer

Public Declare Function obtu_off _
  Lib "AUDINE.DLL" (ByVal base As Integer) As Integer


'===========================
' Acquisition of a CCD image
'===========================
KAF = 1 'KAF-0400 CCD (if KAF=2 -> KAF-1200)
port = get_port(n)  ' get port adress

r = fast_vidage_inv(port, KAF) ' clear image area
r = fast_vidage_inv(port, KAF) ' two time ! 

r = obtu_on(port)  ' open the shutter
r = ampli_off(port)  ' CCD internal amplifier off


' Integration time procedure
' ...................
' .... your code ....
' ...................

r = obtu_off(port)  ' close the shutter - end of exposure

' some delay (200 msec typ. depanding of the shutter machanism)

r = ampli_on(port)  ' CCD internal amplifier on

' some delay (100 msec typ.)

' Read the image
if bin=1 then
   imax = KAF * 768 + overscan
   jmax = KAF * 512
   ReDim buffer(imax - 1, jmax - 1) As Integer ' allocate image buffer
   r = read_win_audine_inv(KAF, 1, imax, 1, jmax, 1, 1, overscan, port, flagsystem, buffer)
else if bin=2 then
   imax = KAF * 384
   jmax = KAF * 256
   ReDim buffer(imax - 1, jmax - 1) As Integer    
   r = read_win_audine_inv(KAF, 1, imax, 1, jmax, 2, 2, 0, port, flagsystem, buffer)
end if
*/

/********************* DLL CODE **************************/

/********************* get_port **************************/
/* Return base adress of the printer port LPTn           */
/*********************************************************/   
_declspec(dllexport) short _stdcall get_port(short n)
{
unsigned short *p;
short port;
p = (unsigned short *) (0x00000408+((n)<<1));
port=(short)*p;
return(port);
}

/******************* read_win_audine_inv ******************/
/* Generic CCD reading routine for Audine                 */
/**********************************************************/
/* KAF=1 -> KAF-0400 type CCD                             */
/* KAF=2 -> KAF-1600 type CCD                             */
/* (x1,y1)-(x2,y2) -> sub-image coordinates               */
/* (binx, biny) -> bin. factors                           */
/* overscan -> overscan value along x-coordinate          */
/* base -> base adress of the printer port                */
/* flagsystem=0 -> 16-bits OS, flagsystem=1 -> 32-bits OS */
/* buf -> image buffer (signed 16-bits)                   */
/**********************************************************/
_declspec(dllexport) short _stdcall read_win_audine_inv(short KAF,
                                    short x1,short x2,
												short y1,short y2,
                                    short binx,short biny,
											   short overscan,short base,
                                    short flagsystem,short *buf)
{
short i,j;
short imax,jmax;
short cx1,cx2;
short P,P2;
short *p0;
int x;
int a1,a2,a3,a4;

P=base;
P2=P+1;
p0=buf;

imax=x2-x1+1;
jmax=y2-y1+1;

cx1=KAF*768+overscan-binx*x2+15;
cx2=KAF*768+overscan-(cx1+binx*imax)+3;

if (flagsystem==0)
   {
   _asm
      {
      cli
      }
   }

/**** skip biny*y1+5 lines ****/
for (i=0;i<biny*y1+5;i++)
   {
   zi_zh_inv(P);
   fast_line_inv(P,KAF);
   fast_line_inv(P,KAF);
   }

for (i=0;i<jmax;i++)
   {
   fast_line_inv(P,KAF);
   zi_zh_inv(P);
   if (biny==2)
      {
      zi_zh_inv(P);
      }
   else if (biny==3)
      {
      zi_zh_inv(P);
      zi_zh_inv(P);
      }
   else if (biny==4)
      {
      zi_zh_inv(P);
      zi_zh_inv(P);
      zi_zh_inv(P);
      }

   /**** skip cx1 first pixels ****/
   for (j=0;j<cx1;j++)
      {
      read_pel_fast_inv(P);
      }

   p0=p0+imax-1;
   if (binx==1)
      {
	   outp(P,247);   // reset
  	   outp(P,255);   
	   outp(P,255);
	   outp(P,255);
      outp(P,255);
	   outp(P,255);
      outp(P,239);   // clamp
      outp(P,255);
      outp(P,251);   // vido
	   outp(P,251);
	   outp(P,251);
	   outp(P,251);
	   outp(P,219);   // start convert

      for (j=0;j<imax;j++)
	     {
	     outp(P,247);   // reset
  	     outp(P,255);   
	     outp(P,255);
	     outp(P,255);
        outp(P,255);
	     outp(P,255);
	     outp(P,239);   // clamp
	     outp(P,255);
        outp(P,251);   // vido
	     outp(P,251);
	     outp(P,251);
	     outp(P,251);
	     outp(P,219);   // start convert

	     // CAN reading
        a1=((inp(P2))>>4)&0x0F;
 	     outp(P,91);  	   	   
	     a2=inp(P2)&0xF0;
 	     outp(P,155);      
	     a3=inp(P2)&0xF0;
 	     outp(P,27);              
	     a4=inp(P2)&0xF0;
 	     x=(a1+a2+(a3<<4)+(a4<<8))^0x8888;      
	     if (x>32767) x=32767;
	     *(p0--)=(short)x;
         }
	   }
   else if (binx==2)
      {
      outp(P,247);   // reset
   	outp(P,255);   
	   outp(P,255);
	   outp(P,255);
      outp(P,255);
	   outp(P,255);
	   outp(P,239);   // clamp
	   outp(P,255);
      outp(P,251);   // vido
  	   outp(P,255);
   	outp(P,251);   // vido
	   outp(P,251);
	   outp(P,251);
	   outp(P,251);
      outp(P,219);   // start convert

      for (j=0;j<imax;j++)
         {
         outp(P,247);   // reset
   	   outp(P,255);   
	      outp(P,255);
	      outp(P,255);
         outp(P,255);
	      outp(P,255);
	      outp(P,239);   // clamp
	      outp(P,255);
         outp(P,251);   // vido
  	      outp(P,255);
   	   outp(P,251);   // vido
	      outp(P,251);
	      outp(P,251);
	      outp(P,251);
         outp(P,219);   // start convert

	      // CAN reading
         a1=((inp(P2))>>4)&0x0F;
 	      outp(P,91);  	   
	      a2=inp(P2)&0xF0;
 	      outp(P,155);      
	      a3=inp(P2)&0xF0;
 	      outp(P,27);              
	      a4=inp(P2)&0xF0;
 	      x=(a1+a2+(a3<<4)+(a4<<8))^0x8888;      

	      if (x>32767) x=32767;
	      *(p0--)=(short)x;
         }
      }
   else if (binx==3)
      {
      outp(P,247);   // reset
  	   outp(P,255);   
	   outp(P,255);
	   outp(P,255);
      outp(P,255);
	   outp(P,255);
	   outp(P,239);   // clamp
	   outp(P,255);
      outp(P,251);   // vido
  	   outp(P,255);
      outp(P,251);   // vido
  	   outp(P,255);
   	outp(P,251);   // vido
	   outp(P,251);
	   outp(P,251);
	   outp(P,251);
      outp(P,219);   // start convert

      for (j=0;j<imax;j++)
         {
         outp(P,247);   // reset
  	      outp(P,255);   
	      outp(P,255);
	      outp(P,255);
         outp(P,255);
	      outp(P,255);
	      outp(P,239);   // clamp
	      outp(P,255);
         outp(P,251);   // vido
  	      outp(P,255);
         outp(P,251);   // vido
  	      outp(P,255);
   	   outp(P,251);   // vido
	      outp(P,251);
	      outp(P,251);
	      outp(P,251);
         outp(P,219);   // start convert

	      // CAN reading
         a1=((inp(P2))>>4)&0x0F;
 	      outp(P,91);  	   
	      a2=inp(P2)&0xF0;
 	      outp(P,155);      
	      a3=inp(P2)&0xF0;
 	      outp(P,27);              
	      a4=inp(P2)&0xF0;
 	      x=(a1+a2+(a3<<4)+(a4<<8))^0x8888;      

 	      if (x>32767) x=32767;
	      *(p0--)=(short)x;
         }
      }
   else if (binx==4)
      {
      outp(P,247);   // reset
  	   outp(P,255);   
	   outp(P,255);
	   outp(P,255);
      outp(P,255);
	   outp(P,255);
	   outp(P,239);   // clamp
	   outp(P,255);
      outp(P,251);   // vido
  	   outp(P,255);
      outp(P,251);   // vido
  	   outp(P,255);
	   outp(P,251);   // vido
  	   outp(P,255);
   	outp(P,251);   // vido
	   outp(P,251);
	   outp(P,251);
	   outp(P,251);
      outp(P,219);   // start convert

      for (j=0;j<imax;j++)
         {
         outp(P,247);   // reset
  	      outp(P,255);   
	      outp(P,255);
	      outp(P,255);
         outp(P,255);
	      outp(P,255);
	      outp(P,239);   // clamp
	      outp(P,255);
         outp(P,251);   // vido
  	      outp(P,255);
         outp(P,251);   // vido
  	      outp(P,255);
	      outp(P,251);   // vido
  	      outp(P,255);
   	   outp(P,251);   // vido
	      outp(P,251);
	      outp(P,251);
	      outp(P,251);
         outp(P,219);   // start convert

	      // CAN reading
         a1=((inp(P2))>>4)&0x0F;
 	      outp(P,91);  	   
	      a2=inp(P2)&0xF0;
 	      outp(P,155);      
	      a3=inp(P2)&0xF0;
 	      outp(P,27);              
	      a4=inp(P2)&0xF0;
 	      x=(a1+a2+(a3<<4)+(a4<<8))^0x8888;      

	      if (x>32767) x=32767;
	      *(p0--)=(short)x;
         }
      }

   p0=p0+imax+1;

   /****  skip cx2 pixels ****/
   for (j=0;j<cx2;j++)
      {
      read_pel_fast_inv(P);
      }
   }

if (flagsystem==0)
   {
   _asm
      {
      sti
      }

   update_clock();
   }

return(0);
}

/********* fast_vidage_inv **********/
/* Fast clear of the CCD            */
/************************************/
_declspec(dllexport) short _stdcall fast_vidage_inv(short base,short KAF)
{
short i,j;

for (i=0;i<KAF*130;i++)
   {
   zi_zh_inv(base);
   zi_zh_inv(base);
   zi_zh_inv(base);
   zi_zh_inv(base);
   for (j=0;j<KAF*790;j++)
      {
      read_pel_fast_inv(base);
      }
   }
return(0);
}

/******* fast_line_inv ********/
/* Fast reading of a CCD line */
/******************************/
void fast_line_inv(short base,short KAF)
{
short j;

for (j=0;j<KAF*790;j++)
   {
   read_pel_fast_inv(base);
   }
}

/***************** zi_zh_inv *******************/
/* Image to horizontal register transfert      */
/***********************************************/
void zi_zh_inv(short base)
{
_asm
      {
	   mov dx,base
      mov al,11111011b
      out dx,al        
      out dx,al
      out dx,al
      out dx,al
      out dx,al
      out dx,al
      out dx,al
      out dx,al
      mov al,11111010b
      out dx,al
      out dx,al
      out dx,al
      out dx,al
      out dx,al
      out dx,al
      out dx,al
      out dx,al
      mov al,11111001b
      out dx,al
      out dx,al
      out dx,al
      out dx,al
      out dx,al
      out dx,al
      out dx,al
      out dx,al
      mov al,11111010b
      out dx,al
      out dx,al
      out dx,al
      out dx,al
      out dx,al
      out dx,al
      out dx,al
      out dx,al
      mov al,11111011b
      out dx,al
      out dx,al
      out dx,al
      out dx,al
      out dx,al
      out dx,al
      out dx,al
      out dx,al
      }
}

/********* read_pel_fast_inv ***********/
/* Fast reading of a pixel             */
/***************************************/
void read_pel_fast_inv(short base)
{

_asm
      {
	   mov dx,base
      mov al,11110111b
      out dx,al
      mov al,11111111b
      out dx,al
      mov al,11111011b
      out dx,al
      }
}  

/********************* TEST_0 **************************/
/* SetDown all the bits of the output port             */
/*******************************************************/
_declspec(dllexport) short _stdcall test_0(short base)
{
outp(base,0);
return(OK);
}

/********************* TEST_255 ************************/
/* SetUp all the bits of the output port               */
/*******************************************************/
_declspec(dllexport) short _stdcall test_255(short base)
{
outp(base,255);
return(OK);
}

/*********************** TEST_C_0 **************************/
/* SetDown all the bits of the control port register       */
/***********************************************************/
_declspec(dllexport) short _stdcall test_c_0(short base)
{
short ctr;
ctr=base+2;
outp(ctr,4); // 0100
return(OK);
}

/********************* TEST_C_255 ************************/
/* SetUp all the bits of the control port register       */
/*********************************************************/
_declspec(dllexport) short _stdcall test_c_255(short base)
{
short ctr;
ctr=base+2;
outp(ctr,11); // 1011
return(OK);
}

/***************** AMPLI_ON *****************/
/* CCD amplifier ON                         */
/********************************************/
_declspec(dllexport) short _stdcall ampli_on(short base)
{
short ctr,x;

ctr=base+2;
x=inp(ctr)&1;
if (x==1)  outp(ctr,inp(ctr)^1);  
return(OK);
}

/*************** AMPLI_OFF ****************/
/* CCD amplifier OFF                      */
/******************************************/
_declspec(dllexport) short _stdcall ampli_off(short base)
{
short ctr,x;

ctr=base+2;
x=inp(ctr)&1;
if (x==0)  outp(ctr,inp(ctr)^1);  
return(OK);
}

/*************** OBTU_ON ******************/
/* Shutter ON                             */
/******************************************/
_declspec(dllexport) short _stdcall obtu_on(short base)
{
short ctr,x;

ctr=base+2;
x=inp(ctr)&2;
if (x==0)  outp(ctr,inp(ctr)^2);  
return(OK);
}

/***************** OBTU_OFF *****************/
/* Shutter OFF                              */
/********************************************/
_declspec(dllexport) short _stdcall obtu_off(short base)
{
short ctr,x;

ctr=base+2;
x=inp(ctr)&2;
if (x==2)  outp(ctr,inp(ctr)^2);  
return(OK);
}

/*************** read_RTC *******************/
/* System date utility                      */
/********************************************/        
int second, minute, hour, day, dayofweek, month, year;

unsigned char read_RTC (int index)
{
outp(CTRL_RTC, index);
return inp(DATA_RTC);
}

void write_RTC (int index, unsigned char value)
{
outp(CTRL_RTC, index);
outp(DATA_RTC, value);
}

/************* read_MC1468818 ***************/
/* System date utility                      */
/********************************************/        
void read_MC146818 (void)
{
if ((read_RTC(REGISTER_D) & MASK_BATTERY) == MASK_BATTERY)
   {
   while ((read_RTC(REGISTER_A) & MASK_UIP) == MASK_UIP); /* attendre la fin de la mise  jour */
   write_RTC((int)REGISTER_B,read_RTC((int)(REGISTER_B) | MASK_UPDATE)); /* bloquer la mise  jour */

   if ((read_RTC(REGISTER_B)&MASK_BCD)==MASK_BCD)
      { /* Mode binaire */
      second    = read_RTC (SECOND);
      minute    = read_RTC (MINUTE);
      hour      = read_RTC (HOUR);
      day       = read_RTC (DAY);
      dayofweek = read_RTC (DAYOFWEEK);
      month     = read_RTC (MONTH);
      year      = read_RTC (YEAR);
      year += 100*BCD2BIN(read_RTC(CENTURY));
      }
   else
      { /* Mode BCD */
      second    = BCD2BIN(read_RTC (SECOND));
      minute    = BCD2BIN(read_RTC (MINUTE));
      hour      = BCD2BIN(read_RTC (HOUR));
      day       = BCD2BIN(read_RTC (DAY));
      dayofweek = BCD2BIN(read_RTC (DAYOFWEEK));
      month     = BCD2BIN(read_RTC (MONTH));
      year      = BCD2BIN(read_RTC (YEAR));
      year += 100*BCD2BIN(read_RTC(CENTURY));
      }          
    write_RTC((int)REGISTER_B,read_RTC((int)(REGISTER_B) & ~MASK_UPDATE)); /* autoriser la mise  jour */
   }
}

/************** uddate_clock ***************/
/* Uddate PC Clock                         */
/*******************************************/ 
void update_clock(void)
{
SYSTEMTIME time;
read_MC146818();

_disable();
GetLocalTime (&time);
time.wSecond = second;
time.wMinute = minute;
time.wHour   = hour;
time.wDay    = day;
time.wDayOfWeek = dayofweek;
time.wMonth  = month;
time.wYear   = year;
SetLocalTime (&time);
_enable();
}